using System;
using System.Collections.Generic;
using System.Text;
using System.Data.OleDb;
using System.Data;
using System.Web.Util;
using System.Net;
using System.Net.Mail;
using System.IO;
using System.Xml;

namespace mailer
{
    class Program
    {
        static void Main(string[] args)
        {
            string configfile = null;

            if (args != null)
            {
                if (args.Length >= 1)
                {
                    configfile = args[0];
                }
            }
            else
            {
                Console.WriteLine("Exit: please set as first argument the file-path for the configuration file");
                return;
            }
            MailerConfig mc = new MailerConfig(configfile);
            if (mc.Ready == false)
            {
                Console.WriteLine("configuration file error - exit:" + mc.ErrMessage);
                return;
            }

            Mailer m = new Mailer(mc);
            
            DataTable dt = m.GetTable();
            foreach(DataRow dr in dt.Rows)
            {
                string strAdress = dr[0].ToString();

                if (m.SendMail(strAdress))
                {
                    Console.WriteLine(strAdress+" was sent");
                }
                else
                {
                    Console.WriteLine("!!! "+strAdress + " " + m.GetErrorMessage );
                }

                //you shouldn't use this program for massive spamming^^
                System.Threading.Thread.Sleep(3000);
            }
        }

    }

    class MailerConfig
    {
        /// <summary>
        /// don't use
        /// </summary>
        private MailerConfig()
        {
        }

        /// <summary>
        /// 
        /// </summary>
        private bool m_bCanStart = false;
        public bool Ready
        {
            get
            {
                return m_bCanStart;
            }
        }

        private string m_strErrMessage="";
        public string ErrMessage
        {
            get
            {
                string tmp = m_strErrMessage;
                m_strErrMessage = "";
                return tmp;
            }
        }

        private string m_strFrom="";
        public string MailFrom
        {
            get
            {
                return m_strFrom;
            }
        }


        private string m_strSubject="";
        public string MailSubject
        {
            get
            {
                return m_strSubject;
            }
        }

        private string m_strBodyFilePath="";
        public string BodyFilePath
        {
            get
            {
                return m_strBodyFilePath;
            }
        }

        private string m_strAttachmentFilePath="";
        public string AttachmentFilePath
        {
            get
            {
                return m_strAttachmentFilePath;
            }
        }

        private string m_strSmtpServer="";
        public string SmtpServer
        {
            get
            {
                return m_strSmtpServer;
            }
        }

        private string m_strPort="";
        public string Port
        {
            get
            {
                return m_strPort;
            }
        }

        private string m_strConnectionString="";
        public string ConnectionString
        {
            get
            {
                return m_strConnectionString;
            }
        }

        private string m_strSelectQry="";
        public string SelectQry
        {
            get
            {
                return m_strSelectQry;
            }
        }

        private string m_strBodyText = "";
        public string MailBody
        {
            get
            {
                return m_strBodyText;
            }
        }

        private System.Net.NetworkCredential m_Credentials;
        public NetworkCredential Credentials
        {
            get
            {
                return m_Credentials;
            }
        }

        private XmlDocument m_xd=new XmlDocument();

        private string GetSingleNodeText(string strXpath)
        {
               return m_xd.SelectSingleNode(strXpath).InnerText;       
        }

        public MailerConfig(string strConfigFileName)
        {
            try
            {
                if (System.IO.File.Exists(strConfigFileName) == false)
                    throw new Exception("File: " + strConfigFileName + " does not exists");

                m_xd.Load(strConfigFileName);

                m_strFrom = GetSingleNodeText("/config/from");

                m_strSubject = GetSingleNodeText("/config/subj");

                m_strBodyFilePath = GetSingleNodeText("/config/bodyfilepath");
                if (System.IO.File.Exists(m_strBodyFilePath) == false)
                {
                    throw new Exception("File: " + m_strBodyFilePath + " does not exists");
                }
                else
                {
                    m_strBodyText= ReadFile(m_strBodyFilePath);
                }

                m_strAttachmentFilePath = GetSingleNodeText("/config/attachmentfilepath");
                if(m_strAttachmentFilePath.Length > 0)
                {
                    if(System.IO.File.Exists(m_strAttachmentFilePath)==false)
                        throw new Exception("File: "+m_strAttachmentFilePath+" does not exists");
                }

                m_strSmtpServer = GetSingleNodeText("/config/smtpserver");
                m_strPort = GetSingleNodeText("/config/port");

//credentials

                string strPW = GetSingleNodeText("/config/password");
                string strUsername = GetSingleNodeText("/config/username");
                string strDomain = GetSingleNodeText("/config/userdomain");
                m_Credentials = new NetworkCredential(strUsername, strPW, strDomain);

                m_strConnectionString = GetSingleNodeText("/config/connectionstring");
                m_strSelectQry = GetSingleNodeText("/config/selectqry");

                m_bCanStart = true;
                
            }
            catch (Exception e)
            {
                m_strErrMessage=e.Message;
            }
        }

        public string ReadFile(string strFileName)
        {
            try
            {
                FileStream fs = new FileStream(strFileName, FileMode.Open, FileAccess.Read);
                StreamReader sr = new StreamReader(fs);

                string ret = sr.ReadToEnd();
                sr.Close(); 
                return ret;
            }
            catch (Exception e)
            {
                m_strErrMessage = "Function ReadFile: " + e.Message;
                return null;
            }
        }
    }

    class Mailer
    {
        private DataTable m_dtTable;
        private MailerConfig m_config;
        private SmtpClient m_SmtpClient;
       
        private Mailer()
        {
        }

        public Mailer(MailerConfig mc)
        {
            m_config = mc;
            
            ReadEmailAddresses();

            m_SmtpClient = new SmtpClient(m_config.SmtpServer, Convert.ToInt32(m_config.Port));
            m_SmtpClient.DeliveryMethod = SmtpDeliveryMethod.Network;

            m_SmtpClient.Credentials = m_config.Credentials;
        }

        private string errMessage = "";
        public string GetErrorMessage
        {
            get
            {
                string tmp = errMessage;
                errMessage = "";
                return tmp;
            }
        }

        public DataTable GetTable()
        {
            return m_dtTable;
        }

        private void ReadEmailAddresses()
        {
            try
            {
                OleDbConnection conn = new OleDbConnection(m_config.ConnectionString);
                OleDbDataAdapter myCommand = new OleDbDataAdapter(m_config.SelectQry, m_config.ConnectionString);//" SELECT * FROM [Tabelle1$] ", strConn);
                DataSet myDataSet = new DataSet();
                myCommand.Fill(myDataSet);

                m_dtTable = myDataSet.Tables[0];
            }
            catch (Exception e)
            {
                errMessage = "Function ReadEmailAddresses: " + e.Message;
            }
        }

        public bool SendMail(string to)
        {
            try
            {
                MailMessage mail = new MailMessage(m_config.MailFrom,to,m_config.MailSubject,m_config.MailBody);

                if(m_config.AttachmentFilePath.Length > 0)
                {
                    Attachment att = new Attachment(m_config.AttachmentFilePath);
                    mail.Attachments.Add(att);
                }

                m_SmtpClient.Send(mail);
               
                return true;
            }
            catch (Exception e)
            {
                errMessage = "Function sendMail: "+ e.Message;
                return false;
            }

        }
    }
}
